/*
 *               ======================================
 *                JMDB_RELEASEDATES.CMD - REXX Example
 *               ======================================
 *
 * Author...........: Juergen Ulbts
 * Website..........: http://www.juergen-ulbts.de/
 * Version..........: 1.9 (2012-01-04)
 *
 * License..........: Public Domain
 *                    Do what you want but you have to follow the IMDb
 *                    data license restrictions.
 *                    Beside that I'm not responsible for any damage
 *                    this script may cause.
 *                    
 * Short Description: Script to create partial HTML data tables with
 *                    movie information imported by the Java Movie
 *                    Database (http://www.jmdb.de/) and extracted
 *                    using this script.
 *                    An example how the result will look like is
 *                    available on my website and also included in the
 *                    sample folder (as complete HTML code).
 *
 * Changelog/History: -------------------------------------------------------------
 *                    1.9 (2012-01-04) - updated special character replacement
 *                                       (HEX: 0xAD, 0xB2 and 0xB3 characters)
 *                    1.8 (2011-12-19) - fixed bug when IMDb IDs are used in the IMDb URL
 *                    1.7 (2011-11-27) - changed title extraction for aka titles and
 *                                       added usage of IMDb IDs for movies (ttxxxxxxx)
 *                    1.6 (2009-12-23) - changed timeframe search to use the SQL
 *                                       BETWEEN command and also changed the
 *                                       date output (console)
 *                    1.5 (2009-06-13) - updated special character replacement
 *                    1.4 (2009-03-20) - updated special character replacement
 *                    1.3 (2009-02-09) - removed additional required library (RexxUtil/RegUtil)
 *                                     - added variables for the filename
 *                    1.2 (2007-05-25) - updated special character replacement
 *                    1.1 (2007-04-16) - updated special character replacement;
 *                                       limited WHERE clause on releasedates even
 *                                       more to remove special releases 
 *                                       (no "director's cut", "premiere", "Festival"
 *                                       AND "Filmfest" releases)
 *                    1.0 (2007-03-15) - first public release
 *
 *
 * =================
 * Long Description:
 * =================
 * This is a sample program (created as HowTo) to extract information from 
 * the movie database tables created by the Java Movie Database application
 * (http://www.jmdb.de/). That application uses data files - so called 
 * LIST files (see 'IMDb Interfaces' on http://www.imdb.com/) - from the
 * Internet Movie Database (IMDb).
 *
 *
 * This script will create only a part of an HTML table with the following
 * columns:
 *  - Title (plus alternative title if available)
 *  - Genre
 *  - Releasedate
 *  - Rank (plus Votes)
 *  - Actors/Actresses (limited to three by default)
 *
 *
 * This example has been created to extract only 'GERMAN' releasedates
 * and will also only extract AKA-Titles used in Germany.
 * To change this you have to slightly modify the SQL-Statements which is
 * very easy.
 * The script will extract movies between to dates based on the current
 * date. First it will search for the last Thursday (movies start normally
 * on Thursdays in Germany) which is the 'base date' used for the date range.
 * From the 'base date' it will go back 2 weeks in the past which is called
 * 'JMDB_FROM_DATE' (see below in the code section). The it will go 3 weeks
 * plus one day into the future from the 'base date'. 
 * This is the 'JMDB_TO_DATE'.
 * Movies have to be between >= JMDB_FROM_DATE and <= JMDB_TO_DATE as
 * one requirement.
 * It's very easy to modify this.
 *
 * Then there should be only movies returned that start in 'Germany' or 
 * 'Worldwide' at the same time. This has been set with the 'JMDB_Country'
 * variable (see below in the code section). If you want to use another
 * country you should look for the name in the Java Movie Database frontend
 * first (open the detail view of a movie).
 *
 * The script will create two files (ATTENTION: hardcoded path!!!).
 * One file contains the partial HTML table data sorted by 'releasedate'
 * and the other one is sorted by 'rank' which shows how good a movie
 * is depending on the votes received.
 *
 * If you want to change the number of actors/actresses shown with the
 * movie you have to modify the variable 'maxAllowedNumberOfActors',
 * which is '3' by default. It has been limited to three so the HTML
 * table won't get to big.
 *
 *
 * This script requires Rexx/SQL and otherwise only uses basic Rexx
 * functions plus the RexxUtil library.
 * Rexx/SQL.........: Rexx/SQL is the SQL package to connect to SQL
 *                    databases like MySQL. It's available as binary
 *                    for OS/2 (eComStation) and Windows. The source
 *                    is also available if you want to build it on other 
 *                    systems.
 *                    http://sourceforge.net/projects/rexxsql/
 *
 * Regina Rexx......: The Regina Rexx implementation if you want to
 *                    run the script on Windows.
 *                    http://sourceforge.net/projects/regina-rexx/
 *
 * RegUtil (Windows): NO LONGER REQUIRED SINCE v1.3 (2009-02-09)
 *                    RegUtil is a port of RexxUtil available on eCS and
 *                    OS/2. It contains several usefull methods.
 *                    Author is Patrick McPhee!
 *                    http://pages.interlog.com/~ptjm/
 *                    or (list of packages)
 *                    http://pages.interlog.com/~ptjm/software.html
 *                      
 * 
 * It has been tested on eComStation (eCS), OS/2 Warp and Windows XP.
 * On Windows XP the Regina Rexx interpreter has been used (together with
 * RegUtil v1.26).
 * The database used was MySQL 5.0.xx on Windows and eComStation (eCS)
 * and OS/2 Warp.
 *
 * 
 * You have to call the script with some parameters to create the partial
 * HTML table. Here is an example how to call the script from normal Rexx
 * or Regina Rexx.
 *
 * Normal Rexx like on OS/2 and eComStation (eCS):
 * ===============================================
 *  jmdb_releasedates.cmd user=<dbuser> pass=<password> data=<name_of_database> serv=<server>
 * 
 * Regina Rexx Interpreter (e.g. on Windows):
 * ==========================================
 *  regina jmdb_releasedates.cmd user=<dbuser> pass=<password> data=<name_of_database> serv=<server>
 *
 * EXAMPLE: MySQL access on a local machine (so no 'serv' parameter):
 * ==================================================================
 *  jmdb_releasedates.cmd user=root pass=root data=jmdb
 *
 * The parameters in detail:
 * =========================
 *  - user: The username needed to access the database
 *  - pass: The password needed to access the database
 *  - data: The name of the database. This should be 'jmdb' which is the default
 *          of the Java Movie Database application.
 *  - serv: The server (IP-Address or hostname of the computer). If it's not given
 *          the Rexx/SQL assumes it's running locally on this computer.
 *
 * If your database does not require values for any of the parameters,
 * simply leave the parameter out.
 *
 *
 * This program will unload the Rexx/SQL external functions on exit,
 * so don't run this if unloading the external functions could cause
 * other running Rexx/SQL programs to fail.
 *
 */
Signal On Syntax
Parse Arg . 'user=' username . 1 . 'pass=' password . 1 . 'data=' database . 1 . 'serv=' server
Call RxFuncAdd 'SQLLoadFuncs', 'rexxsql', 'SQLLoadFuncs' /* Load SQL library */
Call SQLLoadFuncs

/*
 * Set full name to output files
 */
RELEASEDATES_BY_DATE='D:\Daten\Java\Java Movie Database\JMDB_Addons\webupdate\cinema_releasedates_reldate.shtml'
RELEASEDATES_BY_RANK='D:\Daten\Java\Java Movie Database\JMDB_Addons\webupdate\cinema_releasedates_rank.shtml'



/*
 * Connect and display some details of the connection
 */
If SQLConnect( 'c1', username, password, database, server ) < 0 Then Abort( 'connecting' )
Say 'Connect succeeded!'

TMP_DATE = DATE('Standard', GetLastThursday() - 14, 'Base') /* find last thursday and go back 2 more weeks */
JMDB_FROM_DATE = TRANSLATE('1234-56-78', TMP_DATE, '12345678')
Say 'From date: '||JMDB_FROM_DATE

TMP_DATE = DATE('Standard', GetLastThursday() + 22, 'Base') /* find last thursday and to 3 weeks + 1 day (22 days) into the future */
JMDB_TO_DATE = TRANSLATE('1234-56-78', TMP_DATE, '12345678')
Say 'To date: '||JMDB_TO_DATE

JMDB_Country = "Germany" /* '%' = movies in every country; 'Germany' = only movies released in germany or 'Worldwide' */

Say 'For country: '||JMDB_Country
say 'Processing SQL Statements (takes some time)...'

SQL_JMDB_RELEASEDATES_BASE = "SELECT DISTINCT movies.title, movies.movieid, movies.imdbid, movies.year, releasedate, addition, rank, votes FROM movies LEFT JOIN (releasedates LEFT JOIN ratings ON releasedates.movieid=ratings.movieid) ON movies.movieid=releasedates.movieid "
SQL_JMDB_WHERE_CLAUSE = "WHERE releasedate BETWEEN '"||JMDB_FROM_DATE||"' AND '"||JMDB_TO_DATE||"' AND (country = '"||JMDB_Country||"' OR country = 'Worldwide') AND title NOT LIKE '"'"%'"' AND title NOT LIKE '%{%' AND title NOT LIKE '%(TV)%' AND title NOT LIKE '%(VG)%' AND title NOT LIKE '%(V)%' AND (releasedates.addition IS NULL OR releasedates.addition NOT LIKE '%(TV%' AND releasedates.addition NOT LIKE '%(DVD%' AND releasedates.addition NOT LIKE '%director%s cut%' AND releasedates.addition NOT LIKE '%Festival%' AND releasedates.addition NOT LIKE '%Filmfest%' AND releasedates.addition NOT LIKE '%premiere%' ) "
SQL_JMDB_ORDER_CLAUSE = "ORDER BY releasedate, title ASC;"
lcLine = ''
lcLine = SQLQuery(SQL_JMDB_RELEASEDATES_BASE||SQL_JMDB_WHERE_CLAUSE, SQL_JMDB_ORDER_CLAUSE)

IF STREAM(RELEASEDATES_BY_DATE, 'C', 'OPEN WRITE REPLACE') == 'READY:' THEN DO
   /* Write content to file */
   rc = lineout(RELEASEDATES_BY_DATE, lcLine)

   /* Close the file */
   call STREAM RELEASEDATES_BY_DATE, 'C', 'CLOSE'
END
ELSE SAY STREAM(RELEASEDATES_BY_DATE, 'D')




SQL_JMDB_ORDER_CLAUSE = "ORDER BY rank DESC;"
lcLine = ''
lcLine = SQLQuery(SQL_JMDB_RELEASEDATES_BASE||SQL_JMDB_WHERE_CLAUSE, SQL_JMDB_ORDER_CLAUSE)


IF STREAM(RELEASEDATES_BY_RANK, 'C', 'OPEN WRITE REPLACE') == 'READY:' THEN DO
   /* Write content to file */
   rc = lineout(RELEASEDATES_BY_RANK, lcLine)

   /* Close the file */
   call STREAM RELEASEDATES_BY_RANK, 'C', 'CLOSE'
END
ELSE SAY STREAM(RELEASEDATES_BY_RANK, 'D')




/*
 * All done, lets get out - Exit and unload functions
 */
If SQLDisconnect( 'c1' ) < 0 Then Abort( 'disconnecting' )
Say 'Disconnect succeeded!'
Call SQLDropFuncs 'UNLOAD'

Return 0



/*
 * Execute prepared SQL query
 */
SQLQuery:
Parse Arg sqlBaseQuery,sqlOrderClause,.
lcLine = ''
lcCRLF = '0A0D'x

/*
say 'SQL-Command about to be executed...: '||sqlBaseQuery||sqlOrderClause 
*/

If SQLPrepare('p1', sqlBaseQuery||sqlOrderClause ) < 0 Then Call Abort 'Preparing sql statement failed...'

If SQLOpen('p1') < 0 Then Call Abort 'Open prepared sql statement failed...'

sqlWhereClause = ''
lcMovieIdList  = ''

Do Forever
   rc = SQLFetch('p1')
   If rc < 0 Then Call Abort 'Fetching data returned from sql statment failed...'
   If rc = 0 Then Leave
   lcTitle = p1.title
   lcIMDbId = p1.imdbid

   lcAkaTitle = ''
   lcMovieIdList = lcMovieIdList || ' ' || p1.movieid

   sqlWhereClause = sqlWhereClause || ' OR movieid = ' || p1.movieid || ''


   lcLine = lcLine || '            <tr>' || lcCRLF
   
   lcLine = lcLine || '                <td class="bordered">' || '<a href="http://www.IMDb.com/' 
   If p1.imdbid == "" Then
     lcLine = lcLine || 'Title?' || URLEncode(lcTitle)
   Else Do
     lcLine = lcLine || 'title/' || lcIMDbId 
   End
   lcLine = lcLine || '" target="JMDB_MovieId_'||p1.movieid||'" title="'||lcTitle||'" class="external"><b>'||lcTitle||'</b></a>#'||p1.movieid||'#</td>' || lcCRLF
   
   lcLine = lcLine || '                <td class="bordered">#GENRE' || p1.movieid || '#</b></td>' || lcCRLF
   lcLine = lcLine || '                <td class="bordered">' || p1.releasedate || '</b></td>' || lcCRLF
   
   If p1.rank == "" Then
     lcLine = lcLine || '                <td class="bordered"></b></td>' || lcCRLF
   Else Do
     lcLine = lcLine || '                <td class="bordered">' || p1.rank || ' (' || p1.votes || ')</b></td>' || lcCRLF
   End

   lcLine = lcLine || '                <td class="bordered">#ACTORS' || p1.movieid || '#</b></td>' || lcCRLF


   lcLine = lcLine || '            </tr>' || lcCRLF

End

If SQLClose('p1') < 0 Then Call Abort 'closing prepared command after fetch failed...'

If SQLDispose('p1') < 0 Then Call Abort 'Disposing failed...'


/* List with MovieIds for direct SQL use; delete first ' OR ' from string */ 
sqlWhereClause = SUBSTR(sqlWhereClause, 5) 

/* ============================================================= */


/* Retrieve the German akatitles for the main title and replace the placeholder with this title. */ 
sqlString2 = "SELECT movieid, title, addition FROM akatitles WHERE ("||sqlWhereClause||") AND addition LIKE '("||JMDB_Country||")%' ORDER BY title ASC;"

If SQLCommand(p2, sqlString2) < 0 Then Call Abort 'Executing SQL command failed...'
Do j = 1 to sqlca.rowcount
   lcLine = ChangeStr('#'||p2.movieid.j||'#', lcLine, '<br />'||p2.title.j||'&nbsp&nbsp<i>'||p2.addition.j||'</i>')
End

/* Remove all '#<movieid>#' tags still left from the HTML output */
Do j = 1 to WORDS(lcMovieIdList)
   lcLine = ChangeStr('#'||Word(lcMovieIdList, j)||'#', lcLine, '')
End

/* ============================================================= */


/*
 * Retrieve the list of genres the movie belongs to.
 * As a movie ofter belongs to more than one genre we need to make sure
 * that all genres found will be added to a string first and finally 
 * replace the placeholder with the list of genres.
 * THIS IS A LITTLE MORE COMPLICATED... ;)
 */ 
sqlString2 = "SELECT movieid, genre FROM genres WHERE ("||sqlWhereClause||") ORDER BY movieid, genre ASC;"

oldMovieId = ''
lcGenreList = ''
If SQLCommand(p2, sqlString2) < 0 Then Call Abort 'Executing SQL command failed...'
Do j = 1 to sqlca.rowcount

   /*
    * Concat genre if oldMovieId has not been set yet or
    * the current movidId (from the result) equals oldMovieId
    */
   If oldMovieId == '' | oldMovieId == p2.movieid.j Then Do
      oldMovieId = p2.movieid.j
      lcGenreList = lcGenreList||p2.genre.j||', '
   End
   Else Do
      /* 
       * There is a new movieId so we need to replace our HTML
       * tag '#GENRE<movieId>#' (this movieId is actually our
       * variable 'oldMovieId') with the lcGenreList IF it is
       * filled with something...
       * First we need to delete the last ', ' from that string
       * where we also have to check the length of the string
       */
      IF lcGenreList \== '' Then Do
         /* Remove ', ' from string */
         lcGenreList = SUBSTR(lcGenreList, 1, LENGTH(lcGenreList)-2)
         
         /* Replace '#GENRE<movieId>#' tag with genre */
         lcLine = ChangeStr('#GENRE'||oldMovieId||'#', lcLine, lcGenreList)
      End
      
      /*
       * We need to reset our variable lcGenreList to save
       * the new genres in it!
       */
      lcGenreList = ''
      lcGenreList = lcGenreList||p2.genre.j||', '
      
      /* Finally we overwrite oldMovieId with the current value */
      oldMovieId = p2.movieid.j      
   End /* IF..ELSE.. - Block */

End /* DO loop over resultSet */

/* Finally make the last replacement of '#GENRE<movieId># with oldMovieId */
IF lcGenreList \== '' Then Do
   /* Remove ', ' from string */
   lcGenreList = SUBSTR(lcGenreList, 1, LENGTH(lcGenreList)-2)
   
   /* Replace '#GENRE<movieId>#' tag with genre */
   lcLine = ChangeStr('#GENRE'||oldMovieId||'#', lcLine, lcGenreList)
End

/* Remove all '#GENRE<movieId>#' tags still left from the HTML output */
Do j = 1 to WORDS(lcMovieIdList)
   lcLine = ChangeStr('#GENRE'||Word(lcMovieIdList, j)||'#', lcLine, '')
End


/* ============================================================= */

/*
 * Retrieve the list of actors that play in one or more movies.
 * As we don't want to list all actors but only the first few
 * we have to process the list, look for:
 *           '<1>' to <max_we_want_to_display>'
 * 
 * Beside that we have to make sure -- as before with the list
 * of genres -- that we replace the correct movie with the
 * selected actor/actresses list. The list has to be sorted by
 * the substring '<number>' where <1> is the most expensive and
 * important actor/actress. The higher the number gets the less
 * important the actor/actress is.
 *
 * We use the so called COMPOUND VARIABLES from the REXX language
 * for this operation, but it's not that trivial as you might
 * think...
 * There is no build-in SORT function for our use so I had to
 * create my own special Sort function. :-|
 *
 * SO: THIS IS COMPLICATED...
 */ 


sqlString2 = "SELECT movieid, name, as_character FROM actors INNER JOIN movies2actors ON actors.actorid=movies2actors.actorid WHERE ("||sqlWhereClause||") ORDER BY movieid ASC;"

oldMovieId   = ''               /* Default value of oldMovieId */
actorCounter = 0                /* Default counter value */
maxAllowedNumberOfActors = 3    /* Maximum actors allowed to be displayed */

If SQLCommand(p2, sqlString2) < 0 Then Call Abort 'Executing SQL command failed...'


Do j = 1 to sqlca.rowcount
/*
say 'oldMovieId....: ' oldMovieId 
say 'currentMovieId: ' p2.movieid.j
*/
   /*
    * Concat genre if oldMovieId has not been set yet or
    * the current movidId (from the result) equals oldMovieId
    */
   IF oldMovieId == '' | oldMovieId == p2.movieid.j Then Do
      oldMovieId = p2.movieid.j
      actorCounter = actorCounter + 1
      actorList.oldMovieId.0 = actorCounter
      actorList.oldMovieId.actorCounter = p2.name.j ' - ' p2.as_character.j
   End
   Else Do
      /* 
       * There is a new movieId so we need to replace our HTML
       * tag '#ACTORS<movieId>#' (this movieId is actually our
       * variable 'oldMovieId') with the lcGenreList IF it is
       * filled with something...
       * First we need to delete the last ', ' from that string
       * where we also have to check the length of the string
       */
      IF actorList.oldMovieId.0 > 0 Then Do
         /* SORT list by using the '<number>' information */
         Call myActorSortRoutine 'actorList.oldMovieId.'
         
         IF actorList.oldMovieId.0 < maxAllowedNumberOfActors Then
            maxValue = actorList.oldMovieId.0
         Else
            maxValue = maxAllowedNumberOfActors
         /* End - IF..ELSE */

         lcActorString = ''
         Do k=1 to maxValue
            /* Output 'ACTOR NAME - AS_CHARACTER' */
            /* lcActorString = lcActorString||actorList.oldMovieId.k||'<br/>'      */

            /* 
             * ALTERNATIVE (which doesn't need that much space...)
             */

            /* Output 'ACTOR NAME' only */
            lcActorString = lcActorString||SUBSTR(actorList.oldMovieId.k, 1, POS('-',actorList.oldMovieId.k)-2)||'<br/>'            
         End

         DROP actorList.oldMovieId.         


         /* Remove last '<br/>' from string */
         lcActorString = SUBSTR(lcActorString, 1, LENGTH(lcActorString)-5)
         lcActorString = ChangeStr(' ', lcActorString, '&nbsp;')  /* Add different space character */

         /* 
          * Replace '#ACTORS<movieId>#' tag with the
          * first actors and the character they are playing.
          */
         lcLine = ChangeStr('#ACTORS'||oldMovieId||'#', lcLine, lcActorString)

      End /* IF */


      /* We overwrite oldMovieId with the current value */
      oldMovieId = p2.movieid.j

      /* 
       * We reset our counter variable because we recount
       * starting from one as oldMovieId has just been changed.
       */
      actorCounter = 1
            
      /*
       * And we need to reset our compound variable actorList
       * to save the new actor information in it and increase the
       * information how many entries there are.
       */
      actorList.oldMovieId.actorCounter = p2.name.j ' - ' p2.as_character.j
      actorList.oldMovieId.0 = actorCounter
      
   End /* IF..ELSE.. - Block */

End /* DO loop over resultSet */


IF actorList.oldMovieId.0 > 0 Then Do
   /* SORT list by using the '<number>' information */
   Call myActorSortRoutine 'actorList.oldMovieId.'
   
   IF actorList.oldMovieId.0 < maxAllowedNumberOfActors Then
      maxValue = actorList.oldMovieId.0
   Else
      maxValue = maxAllowedNumberOfActors
   /* End - IF..ELSE */

   lcActorString = ''
   Do k=1 to maxValue
      /* Output 'ACTOR NAME - AS_CHARACTER' */
      /* lcActorString = lcActorString||actorList.oldMovieId.k||'<br/>'      */

      /* 
       * ALTERNATIVE (which doesn't need that much space...)
       */

      /* Output 'ACTOR NAME' only */
      lcActorString = lcActorString||SUBSTR(actorList.oldMovieId.k, 1, POS('-',actorList.oldMovieId.k)-2)||'<br/>'            
   End

   /* Remove last '<br/>' from string */
   lcActorString = SUBSTR(lcActorString, 1, LENGTH(lcActorString)-5)
   lcActorString = ChangeStr(' ', lcActorString, '&nbsp;')  /* Add different space character */

   /* 
    * Replace '#ACTORS<movieId>#' tag with the
    * first actors and the character they are playing.
    */
    lcLine = ChangeStr('#ACTORS'||oldMovieId||'#', lcLine, lcActorString)

End

/* Remove all '#ACTORS<movieId>#' tags still left from the HTML output */
Do j = 1 to WORDS(lcMovieIdList)
   lcLine = ChangeStr('#ACTORS'||Word(lcMovieIdList, j)||'#', lcLine, '')
End





/* ============================================================= */
/*
 * Replace special characters with HTML equivalent strings.
 *
 * WARNING: This is not complete there are more characters
 *          that have to be replaced. I added only the most
 *          important ones for myself!!!
 *
 * Changelog:
 * -----------
 * J.U. - 2007-02-01 - first implementation
 * J.U. - 2007-04-16 - updated supported character list
 * J.U. - 2009-03-20 - updated suuported character list
 */
lcLine = ChangeStr('& ', lcLine, '&amp;') /* added one space to be able to use '&nbsp;' above */

lcLine = ChangeStr('', lcLine, '&deg;')

lcLine = ChangeStr('', lcLine, '&euro;')

lcLine = ChangeStr('', lcLine, '&szlig;')

lcLine = ChangeStr('', lcLine, '&auml;')
lcLine = ChangeStr('', lcLine, '&Auml;')

lcLine = ChangeStr('', lcLine, '&euml;')
lcLine = ChangeStr('', lcLine, '&Euml;')

lcLine = ChangeStr('', lcLine, '&iuml;')
lcLine = ChangeStr('', lcLine, '&Iuml;')

lcLine = ChangeStr('', lcLine, '&ouml;')
lcLine = ChangeStr('', lcLine, '&Ouml;')

lcLine = ChangeStr('', lcLine, '&uuml;')
lcLine = ChangeStr('', lcLine, '&Uuml;')

lcLine = ChangeStr('', lcLine, '&yuml;')

/* Special '' encoded as HEX '0xAD' (decimal code: 137) NOT LIKE the standard '-' which is HEX '0x2D'. */
/* see also: http://www.fileformat.info/info/unicode/char/ad/index.htm */
lcLine = ChangeStr('', lcLine, '&#xad;')

/* Special '' encoded as HEX '0xB2' (decimal code: 178). */
/* see also: http://www.fileformat.info/info/unicode/char/b2/index.htm */
lcLine = ChangeStr('', lcLine, '&sup2;')

/* Special '' encoded as HEX '0xB3' (decimal code: 179). */
/* see also: http://www.fileformat.info/info/unicode/char/b3/index.htm */
lcLine = ChangeStr('', lcLine, '&sup3;')


lcLine = ChangeStr('', lcLine, '&agrave;')
lcLine = ChangeStr('', lcLine, '&Agrave;')

lcLine = ChangeStr('', lcLine, '&egrave;')
lcLine = ChangeStr('', lcLine, '&Egrave;')

lcLine = ChangeStr('', lcLine, '&igrave;')
lcLine = ChangeStr('', lcLine, '&Igrave;')

lcLine = ChangeStr('', lcLine, '&ograve;')
lcLine = ChangeStr('', lcLine, '&Ograve;')

lcLine = ChangeStr('', lcLine, '&ugrave;')
lcLine = ChangeStr('', lcLine, '&Ugrave;')


lcLine = ChangeStr('', lcLine, '&aacute;')
lcLine = ChangeStr('', lcLine, '&Aacute;')

lcLine = ChangeStr('', lcLine, '&eacute;')
lcLine = ChangeStr('', lcLine, '&Eacute;')

lcLine = ChangeStr('', lcLine, '&iacute;')
lcLine = ChangeStr('', lcLine, '&Iacute;')

lcLine = ChangeStr('', lcLine, '&oacute;')
lcLine = ChangeStr('', lcLine, '&Oacute;')

lcLine = ChangeStr('', lcLine, '&uacute;')
lcLine = ChangeStr('', lcLine, '&Uacute;')

lcLine = ChangeStr('', lcLine, '&yacute;')
lcLine = ChangeStr('', lcLine, '&Yacute;')


lcLine = ChangeStr('', lcLine, '&acirc;')
lcLine = ChangeStr('', lcLine, '&Acirc;')

lcLine = ChangeStr('', lcLine, '&ecirc;')
lcLine = ChangeStr('', lcLine, '&Ecirc;')

lcLine = ChangeStr('', lcLine, '&icirc;')
lcLine = ChangeStr('', lcLine, '&Icirc;')

lcLine = ChangeStr('', lcLine, '&ocirc;')
lcLine = ChangeStr('', lcLine, '&Ocirc;')

lcLine = ChangeStr('', lcLine, '&ucirc;')
lcLine = ChangeStr('', lcLine, '&Ucirc;')


lcLine = ChangeStr('', lcLine, '&atilde;')
lcLine = ChangeStr('', lcLine, '&Atilde;')

lcLine = ChangeStr('', lcLine, '&ntilde;')
lcLine = ChangeStr('', lcLine, '&Ntilde;')

lcLine = ChangeStr('', lcLine, '&otilde;')
lcLine = ChangeStr('', lcLine, '&Otilde;')


lcLine = ChangeStr('', lcLine, '&oslash;')
lcLine = ChangeStr('', lcLine, '&Oslash;')

lcLine = ChangeStr('', lcLine, '&ccedil;')
lcLine = ChangeStr('', lcLine, '&Ccedil;')

lcLine = ChangeStr('', lcLine, '&aring;')
lcLine = ChangeStr('', lcLine, '&Aring;')

lcLine = ChangeStr('', lcLine, '&aelig;')
lcLine = ChangeStr('', lcLine, '&AElig;')

lcLine = ChangeStr('', lcLine, '&thorn;')
lcLine = ChangeStr('', lcLine, '&THORN;')

lcLine = ChangeStr('', lcLine, '&eth;')
lcLine = ChangeStr('', lcLine, '&ETH;')



/* 
 * The following replacement Collides with the HTML 
 * value definition which also uses the Quote character
 * and has been removed because of that - Example: 'class="bordered"'
 */
/* lcLine = ChangeStr('"', lcLine, '&quot;') */ 
/* ============================================================= */


Return lcLine






/*
 * Returns for a given Date (in 'BASE' format) or the current date the last 
 * date in 'Base' format which is a Thursday.
 *
 * NOTE: In Germany new movies start Thursdays this is why I seek a 'Thursday'!
 */
GetLastThursday:
Parse Arg baseDateToProcess .

/* say 'baseDateToProcess' baseDateToProcess */

/* If there is no parameter use the current date otherwise use the date specified! */
If baseDateToProcess == '' Then Do
   baseDateToProcess = Date('Base')
End

dayOfWeek = Date('Base', baseDateToProcess, 'Base')//7

daysToDecrease = WORD('4 5 6 0 1 2 3', dayOfWeek + 1)   /* This specifies how many days you have to subtract from the current date to find a 'Thursday' */
/* say 'daysToDecrease=' daysToDecrease */
/* say 'baseDateToProcess=' baseDateToProcess */

newDate = DATE('Base', baseDateToProcess-daysToDecrease, 'Base')
/* say 'newDate' newDate */

Return newDate





/*
 * My own actor sort routine using '<number>' to sort
 *
 * NOTE: The proprietary SORT function from reginald would be a nice replacement...
 *
 * Changelog:
 * -----------
 * J.U. - 2007-03-09 - first implementation
 *
 */
myActorSortRoutine:
   /* Add the last '.' to the compound variable if not present */
   _stm = ARG(1) ; if RIGHT(_stm, 1) \= "." then ; _stm = _stm || "."

   _size = VALUE(_stm||"0")
   _swp = 1

   Do while _swp
      _swp = 0
      Do _i = 2 to _size
         _n = _i - 1

         /*
          * GET number from '<number>'
          */
         leftBracketFromN = LASTPOS('<', VALUE(_stm||_n))
         rightBracketFromN = LASTPOS('>', VALUE(_stm||_n))
         valueFromN = 9999 /* Default - Should be high enough for an overall max value */
         IF leftBracketFromN \== 0 & leftBracketFromN \== 0 Then Do
            valueFromN = SUBSTR(VALUE(_stm||_n), leftBracketFromN+1, rightBracketFromN-leftBracketFromN-1)
         END /* END IF..Do */

         /*
          * GET number from '<number>'
          */
         leftBracketFromI = LASTPOS('<', VALUE(_stm||_i))
         rightBracketFromI = LASTPOS('>', VALUE(_stm||_i))
         valueFromI = 9999 /* Default - Should be high enough for an overall max value */
         IF leftBracketFromI \== 0 & leftBracketFromI \== 0 Then Do
            valueFromI = SUBSTR(VALUE(_stm||_i), leftBracketFromI+1, rightBracketFromI-leftBracketFromI-1)
         END /* END IF..Do */

         /*
          * Finally swap if needed....
          */
         IF VALUE(valueFromN) > VALUE(valueFromI) Then Do
           _swp = 1 ; _tmp = VALUE(_stm||_i)
           call VALUE _stm||_i, VALUE(_stm||_n) ; call VALUE _stm||_n, _tmp
         END /* IF Then Do */

      END /* End */
   END /* End */

   /* Cleanup */
   DROP _stm _size _swp _tmp _n leftBracketFromI rightBracketFromI valueFromI leftBracketFromN rightBracketFromN valueFromN
Return





/* 
 * Changestr for Classic REXX!!!
 * This is a fixed version of the version I found on the internet
 * as that implementation was broken (error in concat at the end 
 * of the 'Else' part).
 */
ChangeStr:
Parse Arg needle,haystack,newneedle,.
temp = Pos(needle,haystack)

Do While temp > 0
 If temp = 1 Then haystack = newneedle||Substr(haystack,1 + Length(temp))
 Else haystack = Left(haystack,temp - 1)||newneedle||Substr(haystack,temp + Length(needle))
 temp = Pos(needle,haystack,(temp + Length(newneedle)))
End
Return haystack




/* 
 * URLEncode a string
 *
 * Changelog:
 * -----------
 * J.U. - 2007-03-06 - first implementation
 *
 */
URLEncode:
Procedure
Parse Arg AString
OkayChars = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ'
NewString=""

Do i = 1 to Length( AString)
   Test = SubStr( AString, i, 1)
   If Pos( Test, OkayChars) > 0
      Then NewString = NewString || Test
   Else Do 
      If Test=" "
         Then NewString=NewString||"+"
      Else NewString = NewString || '%' || c2x( Test)
   End
End
Return NewString




/*
 * ABORT ERROR - Abort and unload functions
 */
Abort: Procedure Expose sqlca.
Parse Arg msg
Say 'Program failed:' msg
Say sqlca.interrm
If Datatype( sqlca.sqlerrm.0 ) = 'NUM' Then
   Do i = 1 To sqlca.sqlerrm.0
      Say '('sqlca.sqlstate.i')' sqlca.sqlerrm.i
   End
Else
   Do
      Say '('sqlca.sqlstate')' sqlca.sqlerrm
   End

Call SQLDropFuncs 'UNLOAD'
Call SysDropFuncs

Exit 1



/*
 * SYNTAX ERROR - Show usage information
 */
Syntax: 
Call Abort 'syntax error on line' sigl
Return
